#include <EEPROM.h>
#include <LiquidCrystal.h>
#include <JC_Button.h>

const float DEFAULT_Low_BAT_level = 3.0;
float Low_BAT_level = DEFAULT_Low_BAT_level;
 
//Current steps with a 3R load (R7)
const int Current[] = {7, 40, 72, 104, 136, 168, 200, 233, 265, 297, 330, 362, 394, 426, 458, 490, 523};
 
const byte RS = 2, EN = 3, D4 = 4, D5 = 5, D6 = 6, D7 = 7;
const byte PWM_Pin = 10;
const byte Speaker = 12;
const int BAT_Pin = A0;
int PWM_Value = 0;
unsigned long Capacity = 0;
int ADC_Value = 0;
float BAT_Voltage = 0;
byte Hour = 0, Minute = 0, Second = 0;
bool calc = false, Done = false;
 
LiquidCrystal lcd(RS, EN, D4, D5, D6, D7);
 
Button UP_Button(A2, 25, false, true);
Button Down_Button(A1, 25, false, true);

Button VoltUP_Button(A4, 25, false, true);
Button VoltDown_Button(A3, 25, false, true);

void setup() {
  pinMode(PWM_Pin, OUTPUT);
  pinMode(Speaker, OUTPUT);
 
  EEPROM.get(0, Low_BAT_level);

  if (isnan(Low_BAT_level))
  {
    Low_BAT_level = DEFAULT_Low_BAT_level;
  }

  analogWrite(PWM_Pin, PWM_Value);
 
  UP_Button.begin();
  Down_Button.begin();
  VoltUP_Button.begin();
  VoltDown_Button.begin();
 
  lcd.setCursor(0, 0);
  lcd.begin(16, 2);
  lcd.print("Battery Capacity");
  lcd.setCursor(0, 1);
  lcd.print("Measurement -MEK");
  delay(3000);
  lcd.clear();
  lcd.print("Load Adj:UP/Down");
  lcd.setCursor(0, 1);
  lcd.print(String(Current[0]) + "mA");
  lcd.setCursor(12, 1);
  lcd.print(Low_BAT_level, 2);
  lcd.print("V");
}
 
void loop() {
  UP_Button.read();
  Down_Button.read();
  VoltUP_Button.read();
  VoltDown_Button.read();
 
  if (UP_Button.wasReleased() && PWM_Value < 80 && calc == false)
  {
    PWM_Value = PWM_Value + 5;
    analogWrite(PWM_Pin, PWM_Value);
    lcd.setCursor(0, 1);
    lcd.print("     ");
    lcd.setCursor(0, 1);
    lcd.print(String(Current[PWM_Value / 5]) + "mA");
    tone(Speaker, 400, 30);
  }
 
  if (Down_Button.wasReleased() && PWM_Value > 1 && calc == false)
  {
    PWM_Value = PWM_Value - 5;
    analogWrite(PWM_Pin, PWM_Value);
    lcd.setCursor(0, 1);
    lcd.print("     ");
    lcd.setCursor(0, 1);
    lcd.print(String(Current[PWM_Value / 5]) + "mA");
    tone(Speaker, 400, 30);
  }
  if (UP_Button.pressedFor(1000) && calc == false)
  {
    tone(Speaker, 800, 100);
    lcd.clear();
    timerInterrupt();
  }
  
  if (VoltUP_Button.wasReleased() && Low_BAT_level < 4.9 && calc == false)
  {
    Low_BAT_level += .1;
    lcd.setCursor(12, 1);
    lcd.print("   ");
    lcd.setCursor(12, 1);
    lcd.print(Low_BAT_level, 2);
    lcd.print("V");
    tone(Speaker, 400, 30);
  }
  
  if (VoltDown_Button.wasReleased() && Low_BAT_level > .6 && calc == false)
  {
    Low_BAT_level -= .1;
    lcd.setCursor(12, 1);
    lcd.print("   ");
    lcd.setCursor(12, 1);
    lcd.print(Low_BAT_level, 2);
    lcd.print("V");
    tone(Speaker, 400, 30);
  }
}
 
void timerInterrupt() {
  calc = true;
  EEPROM.put(0, Low_BAT_level);
  while (Done == false) {
    Second ++;
    if (Second == 60) {
      Second = 0;
      Minute ++;
      lcd.clear();
    }
    if (Minute == 60) {
      Minute = 0;
      Hour ++;
    }
    lcd.setCursor(0, 0);
    lcd.print((Hour < 10 ? "0" : "") + String(Hour) + ":" + (Minute < 10 ? "0" : "") + String(Minute) + ":" + (Second < 10 ? "0" : "") + String(Second));
    lcd.setCursor(11, 0);
    ADC_Value = analogRead(BAT_Pin);
    BAT_Voltage = ADC_Value * (5.0 / 1024);
    lcd.print(String(BAT_Voltage) + "V");
    lcd.setCursor(0, 1);
    lcd.print("BAT-C: Wait!...");
 
    if (BAT_Voltage < Low_BAT_level)
    {
      lcd.setCursor(0, 1);
      lcd.print("                ");
      lcd.setCursor(0, 1);
      Capacity =  (Hour * 3600) + (Minute * 60) + Second;
      Capacity = (Capacity * Current[PWM_Value / 5]) / 3600;
      lcd.print("BAT-C:" + String(Capacity) + "mAh");
      Done = true;
      PWM_Value = 0;
      analogWrite(PWM_Pin, PWM_Value);
      tone(Speaker, 800, 2000);
    }
 
    delay(1000);
  }
}